#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

out vec4 color;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform sampler2D texture_diffuse1;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;

float maxCellShadingLevels = 4;

vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation, bool celShading) {
	float d = length(l);
	l = normalize(l);

	float nDotL = dot(n, l);

	float brightness = 1.0;
	if (celShading) {
		// calculate brightness
		brightness = max(nDotL, 0.0);

		// calculate level
		float shadingLevel = floor(brightness * maxCellShadingLevels);

		// set to lower limit of level
		brightness = shadingLevel / maxCellShadingLevels;
	}

	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return ((diffuseF * diffuseC * brightness + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att);
}

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	
	// use in contouring algorithm
	vec3 texColor = texture(texture_diffuse1, vert.uv).rgb;
	color = vec4(texColor * (materialCoefficients.x - 0.3f), 1); // ambient 

	// add directional light contribution
	color.rgb += phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y + 0.2f, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0), true);
			
	// add point light contribution
	color.rgb += phong(n, pointL.position - vert.position_world, v, pointL.color * texColor, materialCoefficients.y + 0.2f, pointL.color, materialCoefficients.z, specularAlpha, true, pointL.attenuation, false);
}

