#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv;
layout(location = 3) in ivec4 boneids;
layout(location = 4) in vec4 weights;

out VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

uniform bool animate;

uniform mat4 modelMatrix;
uniform mat4 viewProjMatrix;
uniform mat3 normalMatrix;

const int MAX_BONES = 100;
uniform mat4 bones[MAX_BONES];

void main() {
	vec4 position_world_;
	if (animate) {
		mat4 bone_transform = bones[boneids[0]] * weights[0];
			bone_transform += bones[boneids[1]] * weights[1];
			bone_transform += bones[boneids[2]] * weights[2];
			bone_transform += bones[boneids[3]] * weights[3];

		position_world_ = modelMatrix * bone_transform * vec4(position, 1.0);
	}
	else {
		position_world_ = modelMatrix * vec4(position, 1.0);
	}
	
	vert.normal_world = normalMatrix * normal;
	vert.uv = uv;
	vert.position_world = position_world_.xyz;

	gl_Position = viewProjMatrix * position_world_;
}